// ***************************************************************************
//
//  pgm:	Interrupt_Test.c
//
//  auth:   Dr. Ing. Ulrich Kaiser ( retired from TID )
//
//  date:    see below
//  version: see below
//
//
//******************************************************************************
//      P10 output      RED LED only
//      P11 output
//		P12 output 
//		P13 input       ADC10	 VBAT+ measurement
//		P14 output
//		P15 output
//		P16 output      green LED
//		P17 output		beeper  
//
// *************************************************************************
//
// UKA20210107 	first version
//				ADC10 measurement is included in measureVbat()
//				  and there is a small window with GIE & CPUOFF.  ADC10
//				  has its own clock; 29 such clocks are used for one conversion.
//				  The green LED demonstrates the conversion rate.
//				Timer1 generates interrupts of 10ms  that are derived from
//				  16MHz DSO clock. The red LED is toggled @ one second rate.
//				Is it possible that the 10ms-interrupts interact in the small
//				  window and bring the CPU to a total hang-up ???
// UKA20210109 After advice from Bruce  today:
//                  green led around critical sequence ;  TA1CCRO to 58 ;
//                  removed the 151ms-delay  ==> higher ADC10 rate ;
//                  changed %100 to %34483 w.r.t. "one second clock" ;
//					experiment with _bic versus _bis i.e. guard on/off   (OK)
// UKA20210111 Made timer1 interrupt even faster : rate 100 kHz
//                 Got failure with   TA1CCRO=20 !!!  need to investigate deeper ...
// UKA20210112 Added option with random delay  ( using srand, rand )
// UKA20210113 Decreased TA1CCR0 from 58 down to 10   ( for j; j-- )
//               Runnning fixed number of ADC10 loops  ( for k )
//              Can switch of random delay easily !
// -------------------------------------------------------------------------
#define  version   	"version 1.3"
#define  date		"13.Jan.2021"

#include <msp430g2553.h>


// P1.0 drives red LED
#define REDLED_OFF   	P1OUT &= ~BIT0
#define REDLED_ON    	P1OUT |= BIT0
#define REDLED_TGL   	P1OUT ^= BIT0

// P1.6 drives green LED 
#define GREENLED_OFF   	P1OUT &= ~BIT6
#define GREENLED_ON    	P1OUT |= BIT6
#define GREENLED_TGL   	P1OUT ^= BIT6

#define BLINKDELAY		25				// 25 ms

#define BEEPER_OFF   	P1OUT &= ~BIT7
#define BEEPER_ON    	P1OUT |= BIT7
#define BEEPER_TGL   	P1OUT ^= BIT7

#define VBAT_FACTOR		3494	// correction 23.11.


// #####  function prototypes   #####
void delay_ms( unsigned int delay );
void blinkGreen( int count );
void blinkRed( int count );
void measureVbat();

void srand( unsigned n );
int  rand();


// the following value is to be measured !
unsigned int Vbat    =  0000; 	//  

// counting the events of 10 ms from Timer_A1
unsigned long timeCounter = 0;	   // 32 bits  !!!



int r;
int j, k;




// ---------------- begin main ------------------------------------------------------
int main(void) 
{
    // Watchdog control
    WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer

///*
	    if (CALBC1_16MHZ ==0xFF || CALDCO_16MHZ == 0xFF)                                     
	     {  
	         while(1);                          // If calibration constants erased
	                                            // do not load, trap CPU!!
	     } 
	    BCSCTL1 = CALBC1_16MHZ;                 // Set DCO to 16MHz
	    DCOCTL = CALDCO_16MHZ;
//*/
#define CLOCKSPEED_MS      16000
#define CLOCKSPEED_US         16
#define clockFrequencyKHZ  16000




	// #####  ADC10 Settings  #####
	//  SREF_1 : V_R+ = VREF+ , V_R- = VSS
	//  ADC10SHT_2 :  S&H is 16 ADC10CLKs long
	//  REFON :  Reference generator ON	    // Vref=1.5V is default, i.e. REF2_5V=0
	//  ADC10ON : ADC10 is ON
	ADC10CTL0 = SREF_1 + ADC10SHT_2 + REFON + ADC10ON;
	// need Vref settling time = 30 us !!!


	// ----- begin PORT 1 -----

	P1DIR |= BIT0;			// Set P1.0 to output direction - launchpad LED red
	REDLED_OFF;				// red LED off !

	// P1.3   INPUT   used for A3  ADC10   (see below)
	
  	P1DIR |= BIT6;			// Set P1.6 to output direction - LED green  and  pulses
	GREENLED_OFF;			// Green LED off !
   	
 	P1DIR |= BIT7;			// set P1.7 to output direction - for BEEPER
    BEEPER_OFF;

  	// ----------- end  Port 1  -----------------
  	
  	
    // ----------------- TIMER_A_1  -----------------------------------
    
  	//TA1CCR0 = 20000;			// period of Timer_A   for 10ms !!!
  	TA1CCR0 =    58;			// period of Timer_A   VERY HIGH EVENT RATE
  	//TA1CCR0 =    20;			// period of Timer_A   VERY HIGH EVENT RATE
 	TA1CTL = TASSEL_2 + MC_1 + ID_3 ;  // SMCLK, up-mode, divide clock by 8
  	TA1CCTL0 = CCIE;            // TA1CCR0 interrupt enabled  (UKA20201129)    
    
    // ----------------- end TIMER_A_1  -------------------------------
    
  	
  	
  	srand( 101 );
  	
  //while(1)		// endless loop !!!
  //{
  
   //for( j=58 ;  j>10 ;  j-- )
   //for( j=100 ;  j>10 ;  j-- )
   for( j=58 ;  j>10 ;  j-- )
   {
   	    TA1CCR0 =    j;		// <===  setting the period of the timer1  !!!
		
		
		for( k=0; k<5000; k++)
		{
			measureVbat();		// contains small window with GIE and CPUOFF !!!
		
			if( 1 )
			{
			   r = rand();
			   r = r % 97;
			   r = abs( r );
        	   while( r-- )
        	   {
            	  __delay_cycles( 1 );
        	   }
			}//if				
		
			//delay_ms( 151 );   //  prime number !!!
		}// for k
		
		TA1CCR0 = 0;  // timer off
		REDLED_OFF;
		delay_ms( 4000 ); // long pause
		
   }//for j

  	
 // }// while(1)    endless loop !
    
}// end ----------- main -----------------------------------------------------------
 	
 	
  	
// #############################################################
// #####    Subroutines for Interrupt Service   (ISR)      #####
// #############################################################

// ADC10 interrupt service routine
//      starts the CPU again after ADC10 has finished
#pragma vector=ADC10_VECTOR

__interrupt void ADC10_ISR(void)
{
    _bic_SR_register_on_exit( CPUOFF );   // Clear CPUOFF bit from (SR)
  
}// ---------------------- ADC10_ISR ----------------------------------------



// Timer A1 interrupt service routine  for overflow of TA1CCR0
#pragma vector=TIMER1_A0_VECTOR
__interrupt void Timer_A1 (void)
{
    timeCounter++;  // overflow of Timer_A1,  i.e. count events of 10 ms
    
    //if( timeCounter %  100000L  == 0 )  REDLED_TGL; // 20210111  CCR0=20
    if( timeCounter %  34483L  == 0 )  REDLED_TGL;	  // new for CCRO=58
    //if( timeCounter %  100L  == 0 )  REDLED_TGL;
    //if( timeCounter %  100L  == 0 )  BEEPER_TGL;
    //BEEPER_TGL;		
         
}// -------------------- Timer_A1 ISR ------------------------------------





// #############################################################
// ### Subroutines for   delay,  i.e. idle cycles of the CPU ###
// #############################################################

void delay_ms( unsigned int delay )
 {
        while (delay--)
        {
            //__delay_cycles( PUT_CPU_CLOCK_SPEED_IN_HZ_DIVIDED_BY_1000_HERE );
            __delay_cycles( CLOCKSPEED_MS );
        }
 }// -----------------------------------------------

// #############################################################
// @@@@@@@  Subroutines for the control of the RG-LED  #########
// #############################################################

void blinkGreen( int count )
{
	int j=999;
	
	if( (count > 0) && (count < 44) )
	{
		for( j=0; j< count; j++ )
		{
			GREENLED_ON	;						
			delay_ms( BLINKDELAY );
			GREENLED_OFF ;							
			delay_ms( BLINKDELAY );
		}//for j
	}// if
}// ---------------- blinkGreen ----------------------


void blinkRed( int count )
{
	int j=999;
	
	if( (count > 0) && (count < 44) )
	{
		for( j=0; j< count; j++ )
		{
			REDLED_ON ;	
			delay_ms( BLINKDELAY );
			REDLED_OFF ;		
			delay_ms( BLINKDELAY );
		}//for j
	}// if
}// ---------------- blinkRed ----------------------




// #############################################################
// ####### Subroutines for the measurements with ADC10 #########
// #############################################################

// measure the  voltage at input A3  ( pin 1.3 )
void measureVbat()
{
	long int value;
	
	ADC10CTL0 &= ~ENC;						// ADC10 disabled
	
  	ADC10CTL1 = INCH_3;                     // input A3
  	ADC10AE0 = BIT3;                        // PA.3 ADC option select
	ADC10CTL0 |=  ADC10IE;     				// interrupt enabled	


	GREENLED_ON;							// <=== monitoring !

	//__bis_SR_register(GIE); 				// Enable global interrupts
	__bic_SR_register(GIE); 				// Disable interrupts to close the window; 
											// 20201218 ;  Bruce McKenney47378
	
	ADC10CTL0 |= ENC + ADC10SC;             // Sampling and conversion start
    __bis_SR_register(CPUOFF + GIE);        // LPM0, ADC10_ISR will force exit from LPM0,  
    										// General Interrupt Enabled !   (open the window)
    										
	GREENLED_OFF;							// <=== monitoring !


	value = (long int)ADC10MEM;
	
	value = value * VBAT_FACTOR / 1000;
	
	Vbat = (unsigned int)value;
	
  	ADC10CTL1 &= ~INCH_3;                   // input A3   off
  	ADC10AE0 &= ~BIT3;                      // PA.3 ADC option select  off
	
	ADC10CTL0 &= ~ENC;						// ADC10 disabled !!!!!!!!!!
} // ------------------------- measureVbat ------------------------------------------

 
